@@ -10,26 +10,54 @@ import string
 import sys
 import time
 from contextlib import closing
+from copy import copy
 
 import requests
 import six
+import dateutil.parser
 from six.moves import range
 
 import koji
 # import parse_arches to current namespace for backward compatibility
 from koji import parse_arches
-from koji.util import to_list
-
-try:
-    import krbV
-except ImportError:  # pragma: no cover
-    krbV = None
+from koji.util import md5_constructor, to_list
 
 
 # for compatibility with plugins based on older version of lib
 # Use optparse imports directly in new code.
+# Nevertheless, TimeOption can be used from here.
 OptionParser = optparse.OptionParser
 
+
+def _check_time_option(option, opt, value):
+    """Converts str timestamp or date/time to float timestamp"""
+    try:
+        ts = float(value)
+        return ts
+    except ValueError:
+        pass
+    try:
+        dt = dateutil.parser.parse(value)
+        ts = time.mktime(dt.timetuple())
+        return ts
+    except Exception:
+        raise optparse.OptionValueError(
+            _("option %s: invalid time specification: %r") % (opt, value))
+
+
+class TimeOption(optparse.Option):
+    """OptionParser extension for timestamp/datetime values"""
+    TYPES = optparse.Option.TYPES + ("time",)
+    TYPE_CHECKER = copy(optparse.Option.TYPE_CHECKER)
+    TYPE_CHECKER['time'] = _check_time_option
+
+    @classmethod
+    def get_help(self):
+        return _("time is specified as timestamp or date/time in any "
+                 "format which can be parsed by dateutil.parser. e.g. "
+                 "\"2020-12-31 12:35\" or \"December 31st 12:35\"")
+
+
 greetings = ('hello', 'hi', 'yo', "what's up", "g'day", 'back to work',
              'bonjour',
              'hallo',
@@ -107,11 +135,15 @@ def get_usage_str(usage):
     return usage + _("\n(Specify the --help global option for a list of other help options)")
 
 
-def ensure_connection(session):
+def ensure_connection(session, options=None):
     try:
         ret = session.getAPIVersion()
-    except requests.exceptions.ConnectionError:
-        error(_("Error: Unable to connect to server"))
+    except requests.exceptions.ConnectionError as ex:
+        warn(_("Error: Unable to connect to server"))
+        if options and getattr(options, 'debug', False):
+            error(str(ex))
+        else:
+            error()
     if ret != koji.API_VERSION:
         warn(_("WARNING: The server is at API version %d and "
                "the client is at %d" % (ret, koji.API_VERSION)))
@@ -144,12 +176,13 @@ def print_task_recurse(task, depth=0):
 
 class TaskWatcher(object):
 
-    def __init__(self, task_id, session, level=0, quiet=False):
+    def __init__(self, task_id, session, level=0, quiet=False, topurl=None):
         self.id = task_id
         self.session = session
         self.info = None
         self.level = level
         self.quiet = quiet
+        self.topurl = topurl
 
     # XXX - a bunch of this stuff needs to adapt to different tasks
 
@@ -203,7 +236,7 @@ class TaskWatcher(object):
         else:
             # First time we're seeing this task, so just show the current state
             if not self.quiet:
-                print("%s: %s" % (self.str(), self.display_state(self.info)))
+                print("%s: %s" % (self.str(), self.display_state(self.info, level=self.level)))
             return False
 
     def is_done(self):
@@ -218,7 +251,7 @@ class TaskWatcher(object):
         state = koji.TASK_STATES[self.info['state']]
         return (state == 'CLOSED')
 
-    def display_state(self, info):
+    def display_state(self, info, level=0):
         # We can sometimes be passed a task that is not yet open, but
         # not finished either.  info would be none.
         if not info:
@@ -230,7 +263,25 @@ class TaskWatcher(object):
             else:
                 return 'open'
         elif info['state'] == koji.TASK_STATES['FAILED']:
-            return 'FAILED: %s' % self.get_failure()
+            s = 'FAILED: %s' % self.get_failure()
+
+            if self.topurl:
+                # add relevant logs if there are any
+                output = list_task_output_all_volumes(self.session, self.id)
+                files = []
+                for filename, volumes in six.iteritems(output):
+                    files += [(filename, volume) for volume in volumes]
+
+                files = [file_volume for file_volume in files if file_volume[0].endswith('log')]
+
+                pi = koji.PathInfo(topdir=self.topurl)
+                # indent more than current level
+                level += 1
+                logs = ['  ' * level + os.path.join(pi.task(self.id, f[1]), f[0]) for f in files]
+                if logs:
+                    s += '\n' + '  ' * level + 'Relevant logs:\n'
+                    s += '\n'.join(logs)
+            return s
         else:
             return koji.TASK_STATES[info['state']].lower()
 
@@ -269,7 +320,7 @@ def display_task_results(tasks):
             print('%s has not completed' % task_label)
 
 
-def watch_tasks(session, tasklist, quiet=False, poll_interval=60, ki_handler=None):
+def watch_tasks(session, tasklist, quiet=False, poll_interval=60, ki_handler=None, topurl=None):
     if not tasklist:
         return
     if not quiet:
@@ -277,7 +328,7 @@ def watch_tasks(session, tasklist, quiet=False, poll_interval=60, ki_handler=Non
     if ki_handler is None:
         def ki_handler(progname, tasks, quiet):
             if not quiet:
-                tlist = ['%s: %s' % (t.str(), t.display_state(t.info))
+                tlist = ['%s: %s' % (t.str(), t.display_state(t.info, level=t.level))
                          for t in tasks.values() if not t.is_done()]
                 print(
                     "Tasks still running. You can continue to watch with the"
@@ -288,7 +339,7 @@ def watch_tasks(session, tasklist, quiet=False, poll_interval=60, ki_handler=Non
     try:
         tasks = {}
         for task_id in tasklist:
-            tasks[task_id] = TaskWatcher(task_id, session, quiet=quiet)
+            tasks[task_id] = TaskWatcher(task_id, session, quiet=quiet, topurl=topurl)
         while True:
             all_done = True
             for task_id, task in list(tasks.items()):
@@ -306,7 +357,7 @@ def watch_tasks(session, tasklist, quiet=False, poll_interval=60, ki_handler=Non
                     child_id = child['id']
                     if child_id not in tasks.keys():
                         tasks[child_id] = TaskWatcher(child_id, session, task.level + 1,
-                                                      quiet=quiet)
+                                                      quiet=quiet, topurl=topurl)
                         tasks[child_id].update()
                         # If we found new children, go through the list again,
                         # in case they have children also
@@ -333,6 +384,7 @@ def bytes_to_stdout(contents):
         sys.stdout.write(contents)
     else:
         sys.stdout.buffer.write(contents)
+        sys.stdout.buffer.flush()
 
 
 def watch_logs(session, tasklist, opts, poll_interval):
@@ -402,7 +454,7 @@ def list_task_output_all_volumes(session, task_id):
     """List task output with all volumes, or fake it"""
     try:
         return session.listTaskOutput(task_id, all_volumes=True)
-    except koji.GenericError as e:
+    except koji.ParameterError as e:
         if 'got an unexpected keyword argument' not in str(e):
             raise
     # otherwise leave off the option and fake it
@@ -529,22 +581,27 @@ def download_file(url, relpath, quiet=False, noprogress=False, size=None,
         # rewrite
         f = open(relpath, 'wb')
 
-    # closing needs to be used for requests < 2.18.0
-    with closing(requests.get(url, headers=headers, stream=True)) as response:
-        if response.status_code == 200:  # full content provided?
-            # rewrite in such case
-            f.close()
-            f = open(relpath, 'wb')
-        response.raise_for_status()
-        length = filesize or int(response.headers.get('content-length') or 0)
-        for chunk in response.iter_content(chunk_size=1024**2):
-            pos += len(chunk)
-            f.write(chunk)
-            if not (quiet or noprogress):
-                _download_progress(length, pos, filesize)
-        if not length and not (quiet or noprogress):
-            _download_progress(pos, pos, filesize)
-    f.close()
+    try:
+        # closing needs to be used for requests < 2.18.0
+        with closing(requests.get(url, headers=headers, stream=True)) as response:
+            if response.status_code in (200, 416):  # full content provided or reaching behind EOF
+                # rewrite in such case
+                f.close()
+                f = open(relpath, 'wb')
+            response.raise_for_status()
+            length = filesize or int(response.headers.get('content-length') or 0)
+            for chunk in response.iter_content(chunk_size=1024**2):
+                pos += len(chunk)
+                f.write(chunk)
+                if not (quiet or noprogress):
+                    _download_progress(length, pos, filesize)
+            if not length and not (quiet or noprogress):
+                _download_progress(pos, pos, filesize)
+    finally:
+        f.close()
+        if pos == 0:
+            # nothing was downloaded, e.g file not found
+            os.unlink(relpath)
 
     if not (quiet or noprogress):
         print('')
@@ -555,19 +612,22 @@ def download_rpm(build, rpm, topurl, sigkey=None, quiet=False, noprogress=False)
     pi = koji.PathInfo(topdir=topurl)
     if sigkey:
         fname = pi.signed(rpm, sigkey)
+        filesize = None
     else:
         fname = pi.rpm(rpm)
+        filesize = rpm['size']
     url = os.path.join(pi.build(build), fname)
     path = os.path.basename(fname)
 
-    download_file(url, path, quiet=quiet, noprogress=noprogress, filesize=rpm['size'])
+    download_file(url, path, quiet=quiet, noprogress=noprogress, filesize=filesize)
 
-    # size
-    size = os.path.getsize(path)
-    if size != rpm['size']:
-        os.unlink(path)
-        error("Downloaded rpm %s size %d does not match db size %d, deleting" %
-              (path, size, rpm['size']))
+    # size - we have stored size only for unsigned copies
+    if not sigkey:
+        size = os.path.getsize(path)
+        if size != rpm['size']:
+            os.unlink(path)
+            error("Downloaded rpm %s size %d does not match db size %d, deleting" %
+                  (path, size, rpm['size']))
 
     # basic sanity
     try:
@@ -598,9 +658,10 @@ def download_archive(build, archive, topurl, quiet=False, noprogress=False):
         url = os.path.join(pi.imagebuild(build), archive['filename'])
         path = archive['filename']
     else:
-        # TODO: cover module/operator-manifests/remote-sources
-        # can't happen
-        assert False  # pragma: no cover
+        # non-legacy types are more systematic
+        directory = pi.typedir(build, archive['btype'])
+        url = os.path.join(directory, archive['filename'])
+        path = archive['filename']
 
     download_file(url, path, quiet=quiet, noprogress=noprogress, filesize=archive['size'])
 
@@ -611,7 +672,7 @@ def download_archive(build, archive, topurl, quiet=False, noprogress=False):
 
     # check checksum/checksum_type
     if archive['checksum_type'] == koji.CHECKSUM_TYPES['md5']:
-        hash = hashlib.md5()
+        hash = md5_constructor()
     elif archive['checksum_type'] == koji.CHECKSUM_TYPES['sha1']:
         hash = hashlib.sha1()
     elif archive['checksum_type'] == koji.CHECKSUM_TYPES['sha256']:
@@ -662,18 +723,6 @@ def warn(msg):
     sys.stderr.flush()
 
 
-def has_krb_creds():
-    if krbV is None:
-        return False
-    try:
-        ctx = krbV.default_context()
-        ccache = ctx.default_ccache()
-        ccache.principal()
-        return True
-    except krbV.Krb5Error:
-        return False
-
-
 def activate_session(session, options):
     """Test and login the session is applicable"""
     if isinstance(options, dict):
@@ -691,7 +740,7 @@ def activate_session(session, options):
             and options.authtype is None:
         # authenticate using user/password
         session.login()
-    elif options.authtype == "kerberos" or has_krb_creds() and options.authtype is None:
+    elif options.authtype == "kerberos" or options.authtype is None:
         try:
             if getattr(options, 'keytab', None) and getattr(options, 'principal', None):
                 session.gssapi_login(principal=options.principal, keytab=options.keytab,
@@ -711,9 +760,15 @@ def _list_tasks(options, session):
     "Retrieve a list of tasks"
 
     callopts = {
-        'state': [koji.TASK_STATES[s] for s in ('FREE', 'OPEN', 'ASSIGNED')],
         'decode': True,
     }
+    if not getattr(options, 'all', False):
+        callopts['state'] = [koji.TASK_STATES[s] for s in ('FREE', 'OPEN', 'ASSIGNED')]
+
+    if getattr(options, 'after', False):
+        callopts['startedAfter'] = options.after
+    if getattr(options, 'before', False):
+        callopts['startedBefore'] = options.before
 
     if getattr(options, 'mine', False):
         if getattr(options, 'user', None):
