@@ -45,7 +45,8 @@ from koji.util import (
     base64encode,
     dslice,
     parseStatus,
-    to_list
+    to_list,
+    joinpath,
 )
 
 
@@ -68,12 +69,12 @@ def incremental_upload(session, fname, fd, path, retries=5, logger=None):
             break
 
         data = base64encode(contents)
-        digest = hashlib.md5(contents).hexdigest()
+        digest = hashlib.sha256(contents).hexdigest()
         del contents
 
         tries = 0
         while True:
-            if session.uploadFile(path, fname, size, digest, offset, data):
+            if session.uploadFile(path, fname, size, ("sha256", digest), offset, data):
                 break
 
             if tries <= retries:
@@ -364,8 +365,8 @@ class SCM(object):
         for allowed_scm in allowed.split():
             scm_tuple = allowed_scm.split(':')
             if len(scm_tuple) < 2:
-                self.logger.warn('Ignoring incorrectly formatted SCM host:repository: %s' %
-                                 allowed_scm)
+                self.logger.warning('Ignoring incorrectly formatted SCM host:repository: %s' %
+                                    allowed_scm)
                 continue
             host_pat = scm_tuple[0]
             repo_pat = scm_tuple[1]
@@ -656,7 +657,8 @@ class TaskManager(object):
             if task_id is None:
                 # not associated with a task
                 # this makes no sense now, but may in the future
-                self.logger.warn("Expiring taskless buildroot: %(id)i/%(tag_name)s/%(arch)s" % br)
+                self.logger.warning("Expiring taskless buildroot: %(id)i/%(tag_name)s/%(arch)s" %
+                                    br)
                 self.session.host.setBuildRootState(id, st_expired)
             elif task_id not in self.tasks:
                 # task not running - expire the buildroot
@@ -692,18 +694,18 @@ class TaskManager(object):
                 data = local_br[id]
                 br = missed_br.get(id)
                 if not br:
-                    self.logger.warn("%(name)s: not in db" % data)
+                    self.logger.warning("%(name)s: not in db" % data)
                     continue
                 desc = "%(id)i/%(tag_name)s/%(arch)s" % br
                 if not br['retire_ts']:
-                    self.logger.warn("%s: no retire timestamp" % desc)
+                    self.logger.warning("%s: no retire timestamp" % desc)
                     continue
                 age = time.time() - br['retire_ts']
                 self.logger.debug("Expired/stray buildroot: %s" % desc)
                 if br and br['task_id']:
                     task = tasks.get(br['task_id'])
                     if not task:
-                        self.logger.warn("%s: invalid task %s" % (desc, br['task_id']))
+                        self.logger.warning("%s: invalid task %s" % (desc, br['task_id']))
                         continue
                     if task['state'] == koji.TASK_STATES['FAILED'] and \
                             age < self.options.failed_buildroot_lifetime:
@@ -723,40 +725,55 @@ class TaskManager(object):
                         if e.errno == errno.ENOENT:
                             rootdir = None
                         else:
-                            self.logger.warn("%s: %s" % (desc, e))
+                            self.logger.warning("%s: %s" % (desc, e))
                             continue
                     else:
                         age = min(age, time.time() - st.st_mtime)
+                topdir_bootstrap = "%s-bootstrap" % topdir
+                if not os.path.exists(topdir_bootstrap):
+                    topdir_bootstrap = None
                 # note: https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=192153)
                 # If rpmlib is installing in this chroot, removing it entirely
                 # can lead to a world of hurt.
                 # We remove the rootdir contents but leave the rootdir unless it
                 # is really old
-                if age > 3600 * 24:
+                if age > self.options.buildroot_final_cleanup_delay:
                     # dir untouched for a day
                     self.logger.info("Removing buildroot: %s" % desc)
-                    if topdir and safe_rmtree(topdir, unmount=True, strict=False) != 0:
+                    if ((topdir and safe_rmtree(topdir, unmount=True, strict=False) != 0) or
+                        (topdir_bootstrap and
+                            safe_rmtree(topdir_bootstrap, unmount=True, strict=False) != 0)):
                         continue
                     # also remove the config
                     try:
                         os.unlink(data['cfg'])
                     except OSError as e:
-                        self.logger.warn("%s: can't remove config: %s" % (desc, e))
-                elif age > 120:
-                    if rootdir:
+                        self.logger.warning("%s: can't remove config: %s" % (desc, e))
+                elif age > self.options.buildroot_basic_cleanup_delay and rootdir:
+                    for d in (topdir, topdir_bootstrap):
+                        if not d:
+                            continue
+                        if d == topdir_bootstrap:
+                            desc2 = "%s [bootstrap]" % desc
+                        else:
+                            desc2 = desc
+                        rootdir = joinpath(d, 'root')
                         try:
                             flist = os.listdir(rootdir)
                         except OSError as e:
-                            self.logger.warn("%s: can't list rootdir: %s" % (desc, e))
+                            self.logger.warning("%s: can't list rootdir: %s" % (desc2, e))
                             continue
                         if flist:
-                            self.logger.info("%s: clearing rootdir" % desc)
+                            self.logger.info("%s: clearing rootdir" % desc2)
                         for fn in flist:
                             safe_rmtree("%s/%s" % (rootdir, fn), unmount=True, strict=False)
-                        resultdir = "%s/result" % topdir
-                        if os.path.isdir(resultdir):
-                            self.logger.info("%s: clearing resultdir" % desc)
-                            safe_rmtree(resultdir, unmount=True, strict=False)
+                        # bootstrap's resultdir is 'results', so we try the best to remove both
+                        # 'result(s)' dirs
+                        for r in ('result', 'results'):
+                            resultdir = "%s/%s" % (d, r)
+                            if os.path.isdir(resultdir):
+                                self.logger.info("%s: clearing resultdir: %s" % (desc2, resultdir))
+                                safe_rmtree(resultdir, unmount=True, strict=False)
                 else:
                     self.logger.debug("Recent buildroot: %s: %i seconds" % (desc, age))
         self.logger.debug("Local buildroots: %d" % len(local_br))
@@ -773,7 +790,7 @@ class TaskManager(object):
             fn = "%s/%s" % (configdir, f)
             if not os.path.isfile(fn):
                 continue
-            fo = open(fn, 'r')
+            fo = koji._open_text_file(fn)
             id = None
             name = None
             for n in range(10):
@@ -916,7 +933,7 @@ class TaskManager(object):
             # note: tasks are in priority order
             self.logger.debug("task: %r" % task)
             if task['method'] not in self.handlers:
-                self.logger.warn("Skipping task %(id)i, no handler for method %(method)s", task)
+                self.logger.warning("Skipping task %(id)i, no handler for method %(method)s", task)
                 continue
             if task['id'] in self.tasks:
                 # we were running this task, but it apparently has been
@@ -1042,8 +1059,8 @@ class TaskManager(object):
                 return True
 
             if t >= timeout:
-                self.logger.warn('Failed to kill %s (pid %i, taskID %i) with signal %i' %
-                                 (execname, pid, task_id, sig))
+                self.logger.warning('Failed to kill %s (pid %i, taskID %i) with signal %i' %
+                                    (execname, pid, task_id, sig))
                 return False
 
             try:
@@ -1072,7 +1089,7 @@ class TaskManager(object):
             proc_path = '/proc/%i/stat' % pid
             if not os.path.isfile(proc_path):
                 return None
-            proc_file = open(proc_path)
+            proc_file = koji._open_text_file(proc_path)
             procstats = [not field.isdigit() and field or int(field)
                          for field in proc_file.read().split()]
             proc_file.close()
@@ -1080,7 +1097,7 @@ class TaskManager(object):
             cmd_path = '/proc/%i/cmdline' % pid
             if not os.path.isfile(cmd_path):
                 return None
-            cmd_file = open(cmd_path)
+            cmd_file = koji._open_text_file(cmd_path)
             procstats[1] = cmd_file.read().replace('\0', ' ').strip()
             cmd_file.close()
             if not procstats[1]:
@@ -1189,7 +1206,7 @@ class TaskManager(object):
         if availableMB < self.options.minspace:
             self.status = "Insufficient disk space at %s: %i MB, %i MB required" % \
                           (br_path, availableMB, self.options.minspace)
-            self.logger.warn(self.status)
+            self.logger.warning(self.status)
             return False
         return True
 
@@ -1260,7 +1277,7 @@ class TaskManager(object):
             raise koji.GenericError("No handler found for method '%s'" % method)
         task_info = self.session.getTaskInfo(task['id'], request=True)
         if task_info.get('request') is None:
-            self.logger.warn("Task '%s' has no request" % task['id'])
+            self.logger.warning("Task '%s' has no request" % task['id'])
             return False
         params = task_info['request']
         handler = handlerClass(task_info['id'], method, params, self.session, self.options)
@@ -1271,15 +1288,15 @@ class TaskManager(object):
                 raise
             except Exception:
                 valid_host = False
-                self.logger.warn('Error during host check')
-                self.logger.warn(''.join(traceback.format_exception(*sys.exc_info())))
+                self.logger.warning('Error during host check')
+                self.logger.warning(''.join(traceback.format_exception(*sys.exc_info())))
             if not valid_host:
                 self.logger.info(
                     'Skipping task %s (%s) due to host check', task['id'], task['method'])
                 return False
         data = self.session.host.openTask(task['id'])
         if data is None:
-            self.logger.warn("Could not open")
+            self.logger.warning("Could not open")
             return False
         task_id = data['id']
         self.tasks[task_id] = data
@@ -1289,11 +1306,11 @@ class TaskManager(object):
         except koji.ActionNotAllowed:
             info2 = self.session.getTaskInfo(task['id'])
             if info2['host_id'] != self.host_id:
-                self.logger.warn("Task %i was reassigned", task_id)
+                self.logger.warning("Task %i was reassigned", task_id)
                 return False
             state = koji.TASK_STATES[info2['state']]
             if state != 'OPEN':
-                self.logger.warn("Task %i changed is %s", task_id, state)
+                self.logger.warning("Task %i changed is %s", task_id, state)
                 return False
             # otherwise...
             raise
@@ -1345,7 +1362,7 @@ class TaskManager(object):
         except koji.xmlrpcplus.Fault as fault:
             response = koji.xmlrpcplus.dumps(fault)
             tb = ''.join(traceback.format_exception(*sys.exc_info())).replace(r"\n", "\n")
-            self.logger.warn("FAULT:\n%s" % tb)
+            self.logger.warning("FAULT:\n%s" % tb)
         except (SystemExit, koji.tasks.ServerExit, KeyboardInterrupt):
             # we do not trap these
             raise
@@ -1355,7 +1372,7 @@ class TaskManager(object):
             return
         except Exception:
             tb = ''.join(traceback.format_exception(*sys.exc_info()))
-            self.logger.warn("TRACEBACK: %s" % tb)
+            self.logger.warning("TRACEBACK: %s" % tb)
             # report exception back to server
             e_class, e = sys.exc_info()[:2]
             faultCode = getattr(e_class, 'faultCode', 1)
