@@ -28,7 +28,6 @@ import random
 import signal
 import time
 
-import requests
 import six.moves.xmlrpc_client
 from six.moves import range
 
@@ -83,7 +82,7 @@ def safe_rmtree(path, unmount=False, strict=True):
             if strict:
                 raise
             else:
-                logger.warn("Error removing: %s", exc_info=True)
+                logger.warning("Error removing: %s", exc_info=True)
                 return 1
         return 0
     if not os.path.exists(path):
@@ -94,7 +93,7 @@ def safe_rmtree(path, unmount=False, strict=True):
     try:
         koji.util.rmtree(path)
     except Exception:
-        logger.warn('file removal failed for %s' % path)
+        logger.warning('file removal failed for %s' % path)
         if strict:
             raise
         return 1
@@ -277,6 +276,12 @@ LEGACY_SIGNATURES = {
     'saveFailedTree': [
         [['buildrootID', 'full'], None, None, (False,)],
     ],
+    'vmExec': [
+        [['name', 'task_info', 'opts'], None, None, None],
+    ],
+    'winbuild': [
+        [['name', 'source_url', 'target', 'opts'], None, None, None],
+    ],
 }
 
 
@@ -495,15 +500,9 @@ class BaseTaskHandler(object):
                 return fn
             self.logger.debug("Downloading %s", relpath)
             url = "%s/%s" % (self.options.topurl, relpath)
-            resp = requests.get(url, stream=True)
-            try:
-                if not os.path.exists(os.path.dirname(fn)):
-                    os.makedirs(os.path.dirname(fn))
-                with open(fn, 'wb') as fdst:
-                    for chunk in resp.iter_content(chunk_size=8192):
-                        fdst.write(chunk)
-            finally:
-                resp.close()
+            if not os.path.exists(os.path.dirname(fn)):
+                os.makedirs(os.path.dirname(fn))
+            koji.downloadFile(url, path=fn)
         else:
             fn = "%s/%s" % (self.options.topdir, relpath)
         return fn
