@@ -23,6 +23,7 @@ from __future__ import absolute_import, division
 import base64
 import calendar
 import datetime
+import errno
 import hashlib
 import logging
 import os
@@ -45,6 +46,18 @@ import koji
 from koji.xmlrpcplus import DateTime
 
 
+# BEGIN kojikamid dup #
+
+def md5_constructor(*args, **kwargs):
+    if hasattr(hashlib._hashlib, 'get_fips_mode') and hashlib._hashlib.get_fips_mode():
+        # do not care about FIPS we need md5 for signatures and older hashes
+        # It is still used for *some* security
+        kwargs['usedforsecurity'] = False
+    return hashlib.md5(*args, **kwargs)
+
+# END kojikamid dup #
+
+
 # imported from kojiweb and kojihub
 def deprecated(message):
     """Print deprecation warning"""
@@ -123,18 +136,18 @@ def duration(start):
     return '%s:%02i' % (mins, secs)
 
 
-def printList(l):
+def printList(lst):
     """Print the contents of the list comma-separated"""
-    if len(l) == 0:
+    if len(lst) == 0:
         return ''
-    elif len(l) == 1:
-        return l[0]
-    elif len(l) == 2:
-        return ' and '.join(l)
+    elif len(lst) == 1:
+        return lst[0]
+    elif len(lst) == 2:
+        return ' and '.join(lst)
     else:
-        ret = ', '.join(l[:-1])
+        ret = ', '.join(lst[:-1])
         ret += ', and '
-        ret += l[-1]
+        ret += lst[-1]
         return ret
 
 
@@ -415,52 +428,129 @@ def lazysetattr(object, name, func, args, kwargs=None, cache=False):
     setattr(object, name, value)
 
 
-def rmtree(path):
+class _RetryRmtree(Exception):
+    """This exception is used internally by rmtree"""
+    # We raise this exception only when it makes sense for rmtree to retry from the top
+
+
+def rmtree(path, logger=None):
     """Delete a directory tree without crossing fs boundaries"""
     # implemented to avoid forming long paths
     # see: https://pagure.io/koji/issue/201
-    st = os.lstat(path)
+    logger = logger or logging.getLogger('koji')
+    try:
+        st = os.lstat(path)
+    except FileNotFoundError:
+        logger.warning("No such file/dir %s for removal" % path)
+        return
     if not stat.S_ISDIR(st.st_mode):
         raise koji.GenericError("Not a directory: %s" % path)
     dev = st.st_dev
     cwd = os.getcwd()
+
     try:
-        os.chdir(path)
-        _rmtree(dev)
+        # retry loop
+        while True:
+            try:
+                os.chdir(path)
+            except OSError as e:
+                if e.errno in (errno.ENOENT, errno.ESTALE):
+                    # likely racing with another rmtree
+                    # if the dir doesn't exist, we're done
+                    return
+                raise
+            try:
+                _rmtree(dev)
+            except _RetryRmtree:
+                # reset and retry
+                os.chdir(cwd)
+                continue
+            break
     finally:
         os.chdir(cwd)
-    os.rmdir(path)
+
+    # a successful _rmtree call should leave us with an empty directory
+    try:
+        os.rmdir(path)
+    except OSError as e:
+        if e.errno != errno.ENOENT:
+            raise
 
 
 def _rmtree(dev):
+    """Remove all contents of CWD"""
+    # This implementation avoids forming long paths and recursion. Otherwise
+    # we will have errors with very deep directory trees.
+    # - to avoid forming long paths we change directory as we go
+    # - to avoid recursion we maintain our own stack
     dirstack = []
+    # Each entry in dirstack is a list of subdirs for that level
+    # As we descend into the tree, we append a new entry to dirstack
+    # When we ascend back up after removal, we pop them off
     while True:
         dirs = _stripcwd(dev)
-        # if no dirs, walk back up until we find some
+
+        # if cwd has no subdirs, walk back up until we find some
         while not dirs and dirstack:
-            os.chdir('..')
+            try:
+                os.chdir('..')
+            except OSError as e:
+                if e.errno in (errno.ENOENT, errno.ESTALE):
+                    # likely in a race with another rmtree
+                    # however, we cannot proceed from here, so we return to the top
+                    raise _RetryRmtree(str(e))
+                raise
             dirs = dirstack.pop()
+
+            # now that we've ascended back up by one, the first dir entry is
+            # one we've just cleared, so we should remove it
             empty_dir = dirs.pop()
             try:
                 os.rmdir(empty_dir)
             except OSError:
-                # we'll still fail at the top level
+                # If this happens, either something else is writing to the dir,
+                # or there is a bug in our code.
+                # For now, we ignore this and proceed, but we'll still fail at
+                # the top level rmdir
                 pass
+
         if not dirs:
             # we are done
             break
-        # otherwise go deeper
+
+        # otherwise we descend into the next subdir
         subdir = dirs[-1]
         # note: we do not pop here because we need to remember to remove subdir later
+        try:
+            os.chdir(subdir)
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                # likely in a race with another rmtree
+                # we'll ignore this and continue
+                # since subdir doesn't exist, we'll pop it off and forget about it
+                dirs.pop()
+                continue  # with dirstack unchanged
+            raise
         dirstack.append(dirs)
-        os.chdir(subdir)
 
 
 def _stripcwd(dev):
     """Unlink all files in cwd and return list of subdirs"""
     dirs = []
-    for fn in os.listdir('.'):
-        st = os.lstat(fn)
+    try:
+        fdirs = os.listdir('.')
+    except OSError as e:
+        # cwd has been removed by others, just return an empty list
+        if e.errno in (errno.ENOENT, errno.ESTALE):
+            return dirs
+        raise
+    for fn in fdirs:
+        try:
+            st = os.lstat(fn)
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                continue
+            raise
         if st.st_dev != dev:
             # don't cross fs boundary
             continue
@@ -528,14 +618,13 @@ def eventFromOpts(session, opts):
     if event_id:
         return session.getEvent(event_id)
     ts = getattr(opts, 'ts')
-    if ts:
+    if ts is not None:
         return session.getLastEvent(before=ts)
     repo = getattr(opts, 'repo')
-    if repo:
-        rinfo = session.repoInfo(repo)
-        if rinfo:
-            return {'id': rinfo['create_event'],
-                    'ts': rinfo['create_ts']}
+    if repo is not None:
+        rinfo = session.repoInfo(repo, strict=True)
+        return {'id': rinfo['create_event'],
+                'ts': rinfo['create_ts']}
     return None
 
 
@@ -583,7 +672,7 @@ def check_sigmd5(filename):
         f.seek(o)
 
         # compute md5 of rest of file
-        md5 = hashlib.md5()
+        md5 = md5_constructor()
         while True:
             d = f.read(1024**2)
             if not d:
@@ -632,7 +721,7 @@ def setup_rlimits(opts, logger=None):
             continue
         if len(limits) == 1:
             limits.append(orig[1])
-        logger.warn('Setting resource limit: %s = %r', key, limits)
+        logger.warning('Setting resource limit: %s = %r', key, limits)
         try:
             resource.setrlimit(rcode, tuple(limits))
         except ValueError as e:
@@ -838,7 +927,7 @@ def parse_maven_chain(confs, scratch=False):
     return builds
 
 
-def to_list(l):
+def to_list(lst):
     """
     Helper function for py2/py3 compatibility used e.g. in
     list(dict.keys())
@@ -847,7 +936,7 @@ def to_list(l):
     used, so it is always an iterator.
     """
 
-    if isinstance(l, list):
-        return l
+    if isinstance(lst, list):
+        return lst
     else:
-        return list(l)
+        return list(lst)
